<?php

namespace WPIS;

class PostTypes {

    use EstatePostMetas;

    static public $postTypeEstateName = 'wpis_estates';
    static public $postTypeProjectName = 'wpis_projects';
    static public $postTypeAgentName = 'wpis_agents';
    static public $postTypeMetaPrefix = 'wpis';
    static private $db;

    private $enableTaxonomies = false;
    private $taxonomies = [
        [
            'slug'         => 'estate_cities',
            'single_name'  => 'Ville',
            'plural_name'  => 'Villes',
            'rewrite'      => ['slug' => 'city'],
        ],
        [
            'slug'         => 'estate_categories',
            'single_name'  => 'Catégorie',
            'plural_name'  => 'Catégories',
            'rewrite'      => ['slug' => 'category'],
        ],
        [
            'slug'         => 'estate_purposes',
            'single_name'  => 'Type de transaction',
            'plural_name'  => 'Types de transaction',
            'rewrite'      => ['slug' => 'type'],
        ],
        [
            'slug'         => 'estate_ammeneties',
            'single_name'  => 'Équipement',
            'plural_name'  => 'Équipements',
            'rewrite'      => ['slug' => 'ameneties'],
        ]
    ];

    private $enableImportSystem = false;
    private $enableImportCustomPostType = false;

    private $enableProjectManagement = false;
    private $enableProjectManagementCustomPostType = false;

    private $enableAgentManagement = false;
    private $enableAgentManagementCustomPostType = false;

    private $postTypeEstateNameCustom = null;

	static public function init() {

        $class = new self();

        self::$db = $GLOBALS['wpdb'];

        $class->checkSettings();

        if( $class->enableImportSystem && !$class->enableImportCustomPostType ){
            add_action( 'init', [$class, 'registerPostTypeEstate']);
            add_action( 'init', [$class, 'handleRedirection' ]);

            add_filter( 'add_menu_classes', [$class, 'displayEstatePostCount']);
            add_filter( 'manage_' . self::$postTypeEstateName .  '_posts_columns', [$class, 'setEstateCustomColumns'] );
            add_action( 'manage_' . self::$postTypeEstateName .  '_posts_custom_column' , [$class, 'applyEstateCustomColumns'], 10, 2 );
        }

        if( $class->enableProjectManagement && !$class->enableProjectManagementCustomPostType ){
            add_action( 'init', [$class, 'registerPostTypeProject']);
            add_action( 'init', [$class, 'enableProjectChildFiltering']);
            add_filter( 'add_menu_classes', [$class, 'displayProjectPostCount']);
            add_action( 'add_meta_boxes', [$class, 'wpisAddMetabox']);
            add_filter( 'manage_' . self::$postTypeProjectName .  '_posts_columns', [$class, 'setProjectCustomColumns'] );
            add_action( 'manage_' . self::$postTypeProjectName .  '_posts_custom_column' , [$class, 'applyProjectCustomColumns'], 10, 2 );
        }

        if( $class->enableAgentManagement && !$class->enableAgentManagementCustomPostType ){
            add_action( 'init', [$class, 'registerPostTypeAgent']);
            add_filter( 'add_menu_classes', [$class, 'displayAgentPostCount']);
            add_filter( 'manage_' . self::$postTypeAgentName .  '_posts_columns', [$class, 'setAgentCustomColumns'] );
            add_action( 'manage_' . self::$postTypeAgentName .  '_posts_custom_column' , [$class, 'applyAgentCustomColumns'], 10, 2 );
        }

        if(
            $class->enableImportSystem && !$class->enableImportCustomPostType ||
            $class->enableProjectManagement && !$class->enableProjectManagementCustomPostType ){

        }{
            add_action( 'add_meta_boxes', [$class, 'wpisAddMetabox']);
            add_action( 'save_post', [$class, 'wpisMetaboxSave'], 10, 2);
        }

        if( $class->enableTaxonomies ){
            add_action( 'init' , [$class, 'registerTaxonomies'] );
        }

        return $class;
    }

    private function checkSettings(){

        $this->enableImportSystem                    = SettingsManagement::get('enableImportSystem');

        $this->enableImportCustomPostType            = SettingsManagement::get('enableImportCustomPostType');
        $this->enableTaxonomies                      = SettingsManagement::get('enableTaxonomies');

        $this->enableProjectManagement               = settingsManagement::get('enableProjectManagement');
        $this->enableProjectManagementCustomPostType = settingsManagement::get('enableProjectManagementCustomPostType');

        $this->enableAgentManagement               = settingsManagement::get('enableAgentManagement');
        $this->enableAgentManagementCustomPostType = settingsManagement::get('enableAgentManagementCustomPostType');

        if( $this->enableImportCustomPostType ){
            $this->postTypeEstateNameCustom = SettingsManagement::get('customPostType');
        }
    }

    public function displayEstatePostCount( $menu ) {
        $count = wp_count_posts(  PostTypes::$postTypeEstateName, 'readable' );
        $count = $count->publish;

        foreach($menu as $index => $item){
            if( $item[2] === 'edit.php?post_type=' . PostTypes::$postTypeEstateName){
                $item[0] .= '<span class="wpis_adminPostCount">' . $count . '</span>' ;
                $menu[$index] = $item;
            }
        }

        return $menu;
    }

    public function displayProjectPostCount( $menu ) {
        $count = wp_count_posts(  PostTypes::$postTypeProjectName, 'readable' );
        $count = $count->publish;

        foreach($menu as $index => $item){
            if( $item[2] === 'edit.php?post_type=' . PostTypes::$postTypeProjectName){
                $item[0] .= '<span class="wpis_adminPostCount">' . $count . '</span>' ;
                $menu[$index] = $item;
            }
        }

        return $menu;
    }

    public function displayAgentPostCount( $menu ) {
        $count = wp_count_posts(  PostTypes::$postTypeAgentName, 'readable' );
        $count = $count->publish;

        foreach($menu as $index => $item){
            if( $item[2] === 'edit.php?post_type=' . PostTypes::$postTypeAgentName){
                $item[0] .= '<span class="wpis_adminPostCount">' . $count . '</span>' ;
                $menu[$index] = $item;
            }
        }

        return $menu;
    }

    public function registerPostTypeEstate(){

        register_post_type(self::$postTypeEstateName,
            [
                'labels'      => [
                    'name'          => __('Propriétés'),
                    'singular_name' => __('Propriété'),
                ],
                'public'      => true,
                'has_archive' => true,
                'menu_icon'   => 'dashicons-admin-home',
                // 'show_in_rest' => true,
                'supports' => array( 'title', 'editor', 'thumbnail' )
            ]
        );

    }

    public function registerPostTypeProject(){

        register_post_type(self::$postTypeProjectName,
            [
                'labels'      => [
                    'name'          => __('Projets'),
                    'singular_name' => __('Projet'),
                ],
                'public'      => true,
                'has_archive' => true,
                'menu_icon'   => 'dashicons-building',
                'supports' => array( 'title', 'editor', 'thumbnail' )
            ]
        );

    }

    public function registerPostTypeAgent(){

        register_post_type(self::$postTypeAgentName,
            [
                'labels'      => [
                    'name'          => __('Agents'),
                    'singular_name' => __('Agent'),
                ],
                'public'      => true,
                'has_archive' => true,
                'menu_icon'   => 'dashicons-admin-users',
                'supports' => array( 'title', 'editor', 'thumbnail' )
            ]
        );

    }

    public function setEstateCustomColumns($columns) {

        $columns['thumbnail'] = __( 'Image', 'wpis' );
        $columns['type-category'] = __( 'Catégorie', 'wpis' );
        $columns['price'] = __( 'Prix', 'wpis' );
        $columns['purpose'] = __( 'Status', 'wpis' );
        $columns['reference'] = __( 'Référence', 'wpis' );
        $columns['locked'] = __( '<i class="fa fa-lock"></i>', 'wpis' );
        $columns['undeletable'] = __( '<i class="fa fa-trash"></i>', 'wpis' );

        $columnClone = $columns;

        $order = [
            'cb',
            'thumbnail',
            'title',
            'type-category',
            'price',
            'purpose',
            'reference',
            'locked',
            'undeletable',
            'date'
        ];

        usort($columns, function ($a, $b) use ($order) {
            if( isset($columns) ){
                $pos_a = array_search($columns[$a], $order);
                $pos_b = array_search($columns[$b], $order);
                return $pos_a - $pos_b;
            }
        });

        $columnsMapped = [];

        foreach( $order as $i => $value ){
            $columnsMapped[$order[$i]] = $columnClone[$value];
        }

        return $columnsMapped;
    }

    public function applyEstateCustomColumns( $column, $post_id ) {
        switch ( $column ) {

            case 'thumbnail' :
                echo get_the_post_thumbnail( $post_id, [50, 50] );
                break;

            case 'type-category' :
                $type = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'category_label' , true );
                $category = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'subcategory_label' , true );
                $typeCategory = self::ucfirstUtf8($type) . ' / ' . self::ucfirstUtf8($category);
                echo $typeCategory;
                break;

            case 'price' :
                $price = round(get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_price' , true ));
                $price = number_format_i18n($price);
                $price = str_replace(',', ' ', $price);
                $currency = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_currency' , true );
                echo $price . ' '  . $currency;
                break;

            case 'purpose' :
                $purpose = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'purpose_label' , true );
                $purpose = self::ucfirstUtf8($purpose);
                echo $purpose;
                break;

            case 'reference' :
                $reference = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'reference' , true );
                echo $reference;
                break;

            case 'locked' :
                $isLocked = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'locked' , true );
                echo $isLocked ? '<i class="icon-lock-on fas fa-lock"></i>' : '<i class="icon-lock-off fas fa-lock-open"></i>';
                break;

            case 'undeletable' :
                $isUndeletable = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'undeletable' , true );
                echo $isUndeletable ? '<i class="icon-undeletable-off fas fa-times-circle"></i>' : '<i class="icon-undeletable-on fas fa-check-circle"></i>';
                break;
        }
    }

    public function setProjectCustomColumns($columns) {

        $columns['thumbnail'] = __( 'Image', 'wpis' );
        $columns['type-category'] = __( 'Catégorie', 'wpis' );
        $columns['price-range'] = __( 'Tranche de prix', 'wpis' );
        $columns['purpose'] = __( 'Status', 'wpis' );
        $columns['reference'] = __( 'Référence', 'wpis' );
        $columns['childsEstateCount'] = __( 'Nombre de biens', 'wpis' );
        $columns['locked'] = __( '<i class="fa fa-lock"></i>', 'wpis' );
        $columns['undeletable'] = __( '<i class="fa fa-trash"></i>', 'wpis' );

        $columnClone = $columns;

        $order = [
            'cb',
            'thumbnail',
            'title',
            'type-category',
            'childsEstateCount',
            'price-range',
            'reference',
            'locked',
            'undeletable',
            'date',
        ];

        usort($columns, function ($a, $b) use ($order) {
            if( isset($columns) ){
                $pos_a = array_search($columns[$a], $order);
                $pos_b = array_search($columns[$b], $order);
                return $pos_a - $pos_b;
            }
        });

        $columnsMapped = [];

        foreach( $order as $i => $value ){
            $columnsMapped[$order[$i]] = $columnClone[$value];
        }

        return $columnsMapped;
    }

    public function setAgentCustomColumns($columns) {

        $columns['thumbnail'] = __( 'Image', 'wpis' );
        $columns['managed_estates'] = __( 'Nombre de biens gérés', 'wpis' );

        $columnClone = $columns;

        $order = [
            'cb',
            'thumbnail',
            'title',
            'managed_estates'
        ];

        usort($columns, function ($a, $b) use ($order) {
            if( isset($columns) ){
                $pos_a = array_search($columns[$a], $order);
                $pos_b = array_search($columns[$b], $order);
                return $pos_a - $pos_b;
            }
        });

        $columnsMapped = [];

        foreach( $order as $i => $value ){
            $columnsMapped[$order[$i]] = $columnClone[$value];
        }

        return $columnsMapped;
    }

    public function applyProjectCustomColumns( $column, $post_id ) {

        $estatePostTypeName =  $this->postTypeEstateNameCustom ?? self::$postTypeEstateName;

        $estateId = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'id' , true );

        $childPosts = query_posts([
            'post_type' => $estatePostTypeName,
            'posts_per_page' => -1,
            'meta_query' => [
                [
                  'key'     => self::$postTypeMetaPrefix . '_' . 'properties_parentId',
                  'compare' => '=',
                  'value'   => $estateId,
                  'type'    => 'CHAR',
                ]
            ]
        ]);

        $childPostsIds = [];
        foreach( $childPosts as $childPost ){
            $childPostsIds[] = $childPost->ID;
        }

        switch ( $column ) {

            case 'thumbnail' :
                echo get_the_post_thumbnail( $post_id, [50, 50] );
                break;

            case 'type-category' :
                $type = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'category_label' , true );
                $typeCategory = self::ucfirstUtf8($type);
                echo $typeCategory;
                break;

            case 'price-range' :
                $price = round(get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_price' , true ));
                $price = number_format_i18n($price);
                $price = str_replace(',', ' ', $price);
                $currency = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_currency' , true );

                if( $childPostsIds ) {
                    $minMax = self::$db->get_results(
                        "
                        SELECT MIN(meta_value) as min, MAX(meta_value) as max FROM (
                            SELECT meta_value FROM " . self::$db->posts . " as posts
                            JOIN " . self::$db->postmeta . " as metas ON posts.ID =  metas.post_id
                            WHERE posts.ID IN (" . implode(',', $childPostsIds) . ")
                            AND metas.meta_key = 'wpis_finance_price'
                            AND post_type  = '" . $estatePostTypeName . "'
                        ) minMaxQuery;
                        "
                    );
                    if( isset($minMax[0]) ){
                        $minPrice = number_format_i18n($minMax[0]->min);
                        $minPrice = str_replace(',', ' ', $minPrice);
                        $maxPrice = number_format_i18n($minMax[0]->max);
                        $maxPrice = str_replace(',', ' ', $maxPrice);
                        echo $minPrice . ' ' . $currency . ' à ' . $maxPrice . ' '  . ($currency ?? '€');
                    }
                }
                else {
                    echo 'N/A';
                }
                break;

            case 'reference' :
                $reference = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'reference' , true );
                echo $reference;
                break;

            case 'locked' :
                $isLocked = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'locked' , true );
                echo $isLocked ? '<i class="icon-lock-on fas fa-lock"></i>' : '<i class="icon-lock-off fas fa-lock-open"></i>';
                break;

            case 'undeletable' :
                $isUndeletable = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'undeletable' , true );
                echo $isUndeletable ? '<i class="icon-undeletable-off fas fa-times-circle"></i>' : '<i class="icon-undeletable-on fas fa-check-circle"></i>';
                break;

            case 'childsEstateCount' :
                echo '<a href="/wp-admin/edit.php?post_type=' . $estatePostTypeName . '&post_parent=' . $post_id . '">';
                echo count($childPosts);
                echo '</a>';
                break;
        }
    }

    public function applyAgentCustomColumns( $column, $post_id ) {

        $estatePostTypeName =  $this->postTypeEstateNameCustom ?? self::$postTypeEstateName;

        $childPosts = query_posts([
            'post_type' => $estatePostTypeName,
            'posts_per_page' => -1,
            'meta_query' => [
                [
                  'key'     => self::$postTypeMetaPrefix . '_' . 'agent_firstname',
                  'compare' => '=',
                  'value'   => get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'agent_firstname' , true ),
                  'type'    => 'CHAR',
                ]
            ]
        ]);

        switch ( $column ) {
            case 'thumbnail' :
                // echo get_the_post_thumbnail( $post_id, [50, 50] );
                $picture = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'agent_picture' , true );
                echo '<img src=' . $picture .' height="50">';
                break;
            case 'managed_estates' :
                echo count($childPosts);
                break;
        }
    }

    public function enableProjectChildFiltering() {
        if ( is_admin() )
            $GLOBALS['wp']->add_query_var( 'post_parent' );
    }

    public function wpisAddMetabox(){

        // Estate Properties metabox
        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] === 'hidden' ){
                continue;
            }
            $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas ' . $group['icon'] . '"></i>%s</span>';
            add_meta_box(
                self::$postTypeMetaPrefix . '_'  . $group['id'],
                sprintf($label, $group['label']),
                function($post) use ($group) {
                    return $this->wpisMetaboxBody($post, $group['id']);
                },
                [self::$postTypeEstateName, self::$postTypeProjectName],
                'side',
                'low',
                ['__back_compat_meta_box' => false]
            );
        }

        // Gallery metabox
        $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas fa-images"></i>%s</span>';
        add_meta_box(
            self::$postTypeMetaPrefix . '_'  . 'pictures',
            sprintf($label, 'Images du bien'),
            [$this, 'wpisMetaboxBodyPictures'],
            [self::$postTypeEstateName, self::$postTypeProjectName],
            'normal',
            'high'
        );
    }

    public function wpisMetaboxBodyPictures( $post ) {
        $medias = get_attached_media('image', $post->ID);

        echo '<ul id="estateMetaboxPictures" class="estateMetabox__pictures">';
        foreach ($medias as $media) {
            echo '<li>';
            echo '  <figure>';
            echo wp_get_attachment_image($media->ID);
            echo '  <i class="fa fa-times"></i>';
            echo '  </figure>';
            echo '</li>';
        }
        echo '</ul>';
        echo "<script>jQuery(function($){

            // Set all variables to be used in scope
            var frame,
                metaBox = $('#estateMetaboxPictures'), // Your meta box id here
                addImgLink = metaBox.find('.upload-custom-img'),
                delImgLink = metaBox.find( '.delete-custom-img'),
                imgContainer = $('#estateMetaboxPictures'),
                imgIdInput = metaBox.find( '.custom-img-id' );

            // ADD IMAGE LINK
            addImgLink.on( 'click', function( event ){

              event.preventDefault();

              // If the media frame already exists, reopen it.
              if ( frame ) {
                frame.open();
                return;
              }

              // Create a new media frame
              frame = wp.media({
                title: 'Select or Upload Media Of Your Chosen Persuasion',
                button: {
                  text: 'Use this media'
                },
                multiple: false  // Set to true to allow multiple files to be selected
              });


              // When an image is selected in the media frame...
              frame.on( 'select', function() {

                // Get media attachment details from the frame state
                var attachment = frame.state().get('selection').first().toJSON();

                // Send the attachment URL to our custom image input field.
                imgContainer.append( '<img src=\"'+attachment.url+'\" style=\"max-width:100%;\"/>' );

                // Send the attachment id to our hidden input
                imgIdInput.val( attachment.id );

                // Hide the add image link
                addImgLink.addClass( 'hidden' );

                // Unhide the remove image link
                delImgLink.removeClass( 'hidden' );
              });

              // Finally, open the modal on click
              frame.open();
            });


            // DELETE IMAGE LINK
            delImgLink.on( 'click', function( event ){

              event.preventDefault();

              // Clear out the preview image
              imgContainer.html( '' );

              // Un-hide the add image link
              addImgLink.removeClass( 'hidden' );

              // Hide the delete image link
              delImgLink.addClass( 'hidden' );

              // Delete the image id from the hidden input
              imgIdInput.val( '' );

            });

          });</script>";
    }

    private function wpisMetaboxBody( $post, $groupId ){
        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] === $groupId ){
                echo '<div class="wpis_options_wrapper">';
                foreach($group['fields'] as $field){
                    $field = str_replace('.', '_', $field);

                    $value = get_post_meta($post->ID, self::$postTypeMetaPrefix . '_' . $field['field'], true);
                    $labelHtml =
                        '<p class="post-attributes-label-wrapper">' .
                        '   <label class="post-attributes-label" for="parent_id">' . $field['label'] . '</label>' .
                        '</p>';

                    if( $field['type'] === 'readonly' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            disabled="disabled"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'text' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'number' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . round($value) .'"
                            type="number"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'date' ) {
                        $value = substr($value, 0, 10);
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="date"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'boolean' ) {
                        echo '<section class="slider-checkbox">';
                        echo '  <input type="checkbox" ' . ($value ? 'checked' : '') . ' id="' . $field['field'] . '" name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '" />';
                        echo '  <label class="label" for="' . $field['field'] . '">' . $field['label'] . '</label>';
                        echo '</section>';
                    }

                    if( $field['type'] === 'file-collection' ) {
                        foreach($value as $file) {
                            echo '<a href="' . $file->url . '" target="_blank">';
                            echo $file->filename . '.' . $file->format;
                            echo '</a>';
                        }
                    }
                }
                echo '</div>';
            }
        }
    }

    public function wpisMetaboxSave( $post_id, $post ) {

        $post_type = get_post_type_object( $post->post_type );

        if ( isset($post_type->cap) && !current_user_can( $post_type->cap->edit_post, $post_id ) )
            return $post_id;

        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] !== 'hidden' ){
                foreach($group['fields'] as $field){
                    if( !empty($_POST) ){
                        $meta = self::$postTypeMetaPrefix . '_' . str_replace('.', '_', $field['field']);
                        if( $field['type'] === 'boolean'){
                            $value = (isset( $_POST[$meta] ) && $_POST[$meta] === 'on') ? 1 : 0;
                            update_post_meta( $post_id, $meta, $value );
                        }
                        elseif( isset( $_POST[$meta] ) ) {
                            update_post_meta( $post_id, $meta, $_POST[$meta] );
                        }
                    }
                }
            }
        }
    }

    public function registerTaxonomies() {
        foreach( $this->taxonomies as $taxonomy ){

            $taxonomy['post_type'] = $this->postTypeEstateNameCustom ?? self::$postTypeEstateName;

            $labels = array(
                'name' => $taxonomy['plural_name'],
                'singular_name' => $taxonomy['single_name'],
                'search_items' =>  'Search ' . $taxonomy['plural_name'],
                'all_items' => 'All ' . $taxonomy['plural_name'],
                'parent_item' => 'Parent ' . $taxonomy['single_name'],
                'parent_item_colon' => 'Parent ' . $taxonomy['single_name'] . ':',
                'edit_item' => 'Edit ' . $taxonomy['single_name'],
                'update_item' => 'Update ' . $taxonomy['single_name'],
                'add_new_item' => 'Add New ' . $taxonomy['single_name'],
                'new_item_name' => 'New ' . $taxonomy['single_name'] . ' Name',
                'menu_name' => $taxonomy['plural_name']
            );

            $rewrite = isset( $taxonomy['rewrite'] ) ? $taxonomy['rewrite'] : array( 'slug' => $taxonomy['slug'] );
            $hierarchical = isset( $taxonomy['hierarchical'] ) ? $taxonomy['hierarchical'] : true;

            register_taxonomy( $taxonomy['slug'], $taxonomy['post_type'], array(
                'hierarchical' => $hierarchical,
                'labels' => $labels,
                'show_ui' => true,
                'query_var' => true,
                'rewrite' => $rewrite,
            ));
        }
    }

    public function handleRedirection() {

        add_action( 'parse_request', function( $wp ){
            if ( preg_match( '#^wpis-estate-id/(.*)#', $wp->request, $matches ) ) {
                $id = $matches[1];

                $result = self::$db->get_row(
                    self::$db->prepare( 'SELECT post_id FROM ' . self::$db->postmeta . ' WHERE meta_key = "wpis_id" AND meta_value = %s',
                    $id
                ));

                if( ! $result ){
                    global $wp_query;
                    $wp_query->set_404();
                    status_header(404);
                    exit;
                }

                $permalink = get_permalink($result->post_id);
                wp_redirect( $permalink, 301 );

                exit;
            }
        } );

    }

    static public function ucfirstUtf8($string){
        if($string == null || !is_string($string) ){
            return false;
        }
        $apresPremier = mb_substr($string,1); //mb_substr : Effectue une opération de type substr() basée sur le nombre de caractères
        $string = trim($string);
        $maj = utf8_decode($string);
        $maj = $maj[0];
        $a = 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ??';
        $b = 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuyybyRr';
        $maj = strtr($maj, utf8_decode($a), $b);
        $string = substr_replace($string, $maj, 0);
        $string = utf8_encode($string);
        $string = ucfirst($string);
        $string = $string.$apresPremier;
        return $string;
    }
}